<?php

class ForteEcomHelper
{
    const HOST = "https://ecom-merchant-api.fortebank.com";
    const HOST_DEMO = "https://qa-merchant-go.fortebank.com";

    const ENDPOINT_INIT = "/init";
    const ENDPOINT_STATUS = "/status";
    const ENDPOINT_COMPLETE = "/complete";
    const ENDPOINT_PURCHASE = "/purchase";
    const ENDPOINT_PURCHASE_TOKEN = "/token/purchase";
    const ENDPOINT_REFUND = "/refund";
    const ENDPOINT_REVERSE = "/reverse";

    const ENDPOINT_RESPONSE_CODE = "https://adminapi.forte.kz/codes";

    const STATUS_INIT = 'CREATED';
    const STATUS_LOCK = 'ON_LOCK';
    const STATUS_PROCESS = 'ON_PAYMENT';
    const STATUS_SUCCESS = 'APPROVED';
    const STATUS_DECLINE = 'DECLINED';
    const STATUS_ON_HOLD = 'PREAUTH_APPROVED';
    const STATUS_REVERSE = 'REVERSED';
    const STATUS_REFUND = 'REFUNDED';
    const STATUS_EXPIRE = 'EXPIRED';
    const STATUS_CANCEL = 'CANCELED';
    const STATUS_ERROR = 'ERROR';

    const AVAILABLE_CURRENCIES = [
        'KZT' => [
            'id'   => 398,
            'code' => 'KZT',
            'name' => 'Kazakhstani tenge'
        ],
    ];

    /**
     * @param            $_forteecom_merchant_id
     * @param            $pre_auth
     * @param            $amount
     * @param            $remote_id
     * @param            $description
     * @param string     $currency_code
     * @param vmPSPlugin $plugin
     *
     * @return array
     */
    public static function init_payment($_forteecom_merchant_id, $pre_auth, $amount, $remote_id, $description, $currency_code = 'KZT', $plugin = null)
    {
        $currency_id = self::get_currency_id($currency_code);
        $data = array(
            'merchantId' => $_forteecom_merchant_id,
            'preAuth'    => $pre_auth,
            'source'     => array(
                'jsonClass'   => 'TransactionSourcePurchase',
                'amount'      => (float)$amount,
                'currency'    => (string)$currency_id,
                'remoteId'    => $remote_id,
                'description' => $description
            )
        );

        $request_body = json_encode($data);
        $request_result = self::send_request(self::ENDPOINT_INIT, $request_body, $plugin);

        $_forteecom_order_id = null;
        $_forteecom_session_id = null;
        if (empty($request_result['error'])) {
            if ($request_result['response']->responseCode == 0) {
                $_forteecom_order_id   = $request_result['response']->data->orderId;
                $_forteecom_session_id = $request_result['response']->data->sessionId;
            }else {
                $request_result['error'] = $request_result['response']->responseMessage;
            }
        }

        return array(
            'error_message'         => $request_result['error'],
            '_forteecom_order_id'   => $_forteecom_order_id,
            '_forteecom_session_id' => $_forteecom_session_id,
        );
    }

    public static function get_currency_id($currency_code)
    {
        return self::AVAILABLE_CURRENCIES[$currency_code]['id'];
    }

    /**
     * @param            $endpoint
     * @param            $body
     * @param vmPSPlugin $plugin
     *
     * @return array
     */
    public static function send_request($endpoint, $body, $plugin = null)
    {
        $curl = curl_init();
        $url = self::get_url($endpoint);
        curl_setopt_array($curl, array(
            CURLOPT_URL            => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_SSL_VERIFYHOST => 0,
            CURLOPT_SSL_VERIFYPEER => 0,
            CURLOPT_ENCODING       => '',
            CURLOPT_MAXREDIRS      => 10,
            CURLOPT_TIMEOUT        => 30,
            CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST  => 'POST',
            CURLOPT_POSTFIELDS     => $body,
            CURLOPT_HTTPHEADER     => array(
                'Content-Type: application/json',
            )
        ));

        $response = json_decode(curl_exec($curl));
        $error = curl_error($curl);
        curl_close($curl);

        if (empty($error) && !property_exists($response, 'responseCode')) {
            $error = $response->moreInfo;
        }

        if (isset($plugin)) {
            $log_data = array(
                'url'      => $url,
                'request'  => $body,
                'response' => $response,
                'error'    => $error,
            );
            $plugin->debugLog(print_r($log_data, true), 'ForteEcom request', 'debug');
        }

        return array(
            'response' => $response,
            'error'    => $error
        );
    }

    /**
     * @param            $_forteecom_merchant_id
     * @param            $_forteecom_order_id
     * @param            $_forteecom_session_id
     * @param vmPSPlugin $plugin
     *
     * @return array
     */
    public static function get_payment_status($_forteecom_merchant_id, $_forteecom_order_id, $_forteecom_session_id, $plugin = null)
    {
        $data = array(
            'merchantId' => $_forteecom_merchant_id,
            'orderId'    => $_forteecom_order_id,
            'sessionId'  => $_forteecom_session_id,
        );

        $request_body = json_encode($data);

        $request_result = self::send_request(self::ENDPOINT_STATUS, $request_body, $plugin);

        $_forteecom_order_status = null;
        $_forteecom_add_info = null;
        if (empty($request_result['error'])) {
            $_forteecom_order_status = $request_result['response']->data->orderStatus;
            $_forteecom_add_info = $request_result['response']->data->addInfo;
        }

        return array(
            'error_message'           => $request_result['error'],
            '_forteecom_order_status' => $_forteecom_order_status,
            '_forteecom_add_info'     => $_forteecom_add_info,
        );
    }

    /**
     * @param          $_forteecom_merchant_id
     * @param          $_forteecom_order_id
     * @param          $_forteecom_session_id
     * @param vmPlugin $plugin
     *
     * @return array
     */
    public static function complete_payment($_forteecom_merchant_id, $_forteecom_order_id, $_forteecom_session_id, $plugin = null)
    {
        $data = array(
            'merchantId' => $_forteecom_merchant_id,
            'orderId'    => $_forteecom_order_id,
            'sessionId'  => $_forteecom_session_id,
        );

        $request_body = json_encode($data);

        $request_result = self::send_request(self::ENDPOINT_COMPLETE, $request_body, $plugin);

        $_forteecom_approval_code = null;
        if (empty($request_result['error'])) {
            $_forteecom_approval_code = $request_result['response']->data->approvalCode;
        }

        return array(
            'error_message'            => $request_result['error'],
            '_forteecom_approval_code' => $_forteecom_approval_code,
        );
    }

    public static function refund_payment($_forteecom_merchant_id, $_forteecom_order_id, $_forteecom_session_id, $amount, $currency_code, $plugin = null)
    {
        $currency_id = self::get_currency_id($currency_code);
        $data        = array(
            'orderIdent' => array(
                'merchantId' => $_forteecom_merchant_id,
                'orderId'    => $_forteecom_order_id,
                'sessionId'  => $_forteecom_session_id,
            ),
            'transaction' => array(
                //'jsonClass'   => "TransactionSourceRefund",
                'amount'      => (float)$amount,
                'currency'    => (string)$currency_id
            ),
        );

        $request_body = json_encode($data);
        $request_result = self::send_request(self::ENDPOINT_REFUND, $request_body, $plugin);
        $_forteecom_message = null;
        if (empty($request_result['error'])) {
            $_forteecom_message = (isset($request_result['response']->message)) ? $request_result['response']->message : 'Success';
        } else {
            $request_result['error'] = 'Error occured on refund';
        }

        return [
            'error_message'      => $request_result['error'],
            '_forteecom_message' => $_forteecom_message,
        ];
    }

    /**
     * @param string     $_forteecom_merchant_id
     * @param string     $_forteecom_order_id
     * @param string     $_forteecom_session_id
     * @param float      $amount
     * @param int        $remote_id
     * @param string     $description
     * @param int        $currency_id
     * @param object     $card
     *
     * @param vmPSPlugin $plugin
     *
     * @return array
     */
    public static function purchase_by_token($_forteecom_merchant_id, $_forteecom_order_id, $_forteecom_session_id, $amount, $remote_id, $description, $currency_id, $card, $plugin = null)
    {
        $data = array(
            'merchantId' => $_forteecom_merchant_id,
            'orderId'    => $_forteecom_order_id,
            'sessionId'  => $_forteecom_session_id,
            'ip'         => $_SERVER['REMOTE_ADDR'],
            'token'      => $card->card_token,
            'source'     => array(
                'jsonClass'   => "TransactionSourcePurchase",
                'amount'      => (float)$amount,
                'currency'    => (string)$currency_id,
                'remoteId'    => $remote_id,
                'description' => $description
            ),
        );

        $request_body = json_encode($data);
        $request_result = self::send_request(self::ENDPOINT_PURCHASE_TOKEN, $request_body, $plugin);

        $_forteecom_order_status = null;
        $_forteecom_order_data = null;
        if (empty($request_result['error'])) {
            $_forteecom_order_status = $request_result['response']->responseMessage;
            if ($request_result['response']->responseCode != 0) {
                $_forteecom_order_data = $request_result['response']->data->purchaseResponseCode;
                $request_result['error'] = self::get_response_message($request_result['response']->data->purchaseResponseCode);
            } else {
                $_forteecom_order_data = $request_result['response']->data->result;
            }
        }

        return array(
            'error_message'           => $request_result['error'],
            '_forteecom_order_status' => $_forteecom_order_status,
            '_forteecom_order_data'   => $_forteecom_order_data,
        );

    }

    /**
     * Get localized message for error
     *
     * @param string $code
     *
     * @return string
     */
    public static function get_response_message($code)
    {
        // todo: get locale
        switch ('') {
            case 'ru_RU':
                $lang = '_ru';
                break;
            case 'kk':
                $lang = '_kk';
                break;
            default:
                $lang = '';
                break;
        }

        $url = self::ENDPOINT_RESPONSE_CODE . '?' . http_build_query(['name' => $code]);

        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL            => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_SSL_VERIFYHOST => 0,
            CURLOPT_SSL_VERIFYPEER => 0,
            CURLOPT_ENCODING       => '',
            CURLOPT_MAXREDIRS      => 10,
            CURLOPT_TIMEOUT        => 30,
            CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1
        ));

        $response = json_decode(curl_exec($curl), true);
        curl_close($curl);

        return $response[0]['message' . $lang];
    }

    /**
     * @param                       $order
     * @param                       $_forteecom_order_id
     * @param                       $_forteecom_order_status
     * @param                       $current_status
     * @param plgVmPaymentForteecom $plugin
     * @param string|null           $button_text
     * @param null                  $query_args
     *
     *
     * @since version
     */
    public static function process_payment_status(&$order, $_forteecom_order_id, $_forteecom_order_status, $current_status, &$plugin = null, &$button_text = null, &$query_args = null)
    {
        if ($current_status == $_forteecom_order_status) {
            return;
        }
        switch ($_forteecom_order_status) {
            case self::STATUS_SUCCESS:
                $order['order_status'] = 'F';
                $cart = VirtueMartCart::getCart();
                $cart->emptyCart();
                break;
            case self::STATUS_EXPIRE:
            case self::STATUS_DECLINE:
                $order['order_status'] = 'D';
                $button_text = vmText::_('CART_PAGE');
                $query_args = array(
                    'option' => 'com_virtuemart',
                    'view'   => 'cart',
                );
                break;
            case self::STATUS_REVERSE:
            case self::STATUS_REFUND:
                $order['order_status'] = 'R';
                $query_args = array(
                    'option' => 'com_virtuemart',
                    'view'   => 'cart',
                );
                break;
            case self::STATUS_CANCEL:
                $order['order_status'] = 'X';
                $query_args = array(
                    'option' => 'com_virtuemart',
                    'view'   => 'cart',
                );
                break;
            case self::STATUS_ON_HOLD:
                $order['order_status'] = 'C';
                break;
        }
    }

    public static function get_available_currency_names()
    {
        $currencies = [];
        foreach (self::AVAILABLE_CURRENCIES as $currency) {
            $currencies[] = $currency['name'];
        }
        return implode(', ', $currencies);
    }

    public static function get_url($endpoint)
    {
        $host = (FORTEECOM_DEMO_ENABLED) ? self::HOST_DEMO : self::HOST;
        return $host . '/ecom' . $endpoint;
    }

    public static function get_script() {
        if (FORTEECOM_DEMO_ENABLED) {
            //$cdn_url = 'https://cdn-1.forte.kz/assets/plugins/forte-ecom-qa/v1/ecom-widget-feature-plugin/';
            $cdn_url = 'https://cdn-1.forte.kz/assets/plugins/forte-ecom-qago/v1/ecom-widget-feature-plugin/';
        } else {
            $cdn_url = 'https://cdn-1.forte.kz/assets/plugins/forte-ecom/v1/ecom-widget-feature-plugin/';
        }

        return '<script type="module" src="' . $cdn_url . 'ecom-widget-feature-plugin.esm.js?ver=' . time() . '"></script>
        <script nomodule="" src="' . $cdn_url . 'ecom-widget-feature-plugin.js?ver=' . time() . '"></script>';
    }

}